searchNodes=[{"doc":"Templates are used by Phoenix when rendering responses. Since many views render significant content, for example a whole HTML file, it is common to put these files into a particular directory, typically &quot;APP_web/templates&quot;. This module provides conveniences for reading all files from a particular directory and embedding them into a single module. Imagine you have a directory with templates: # templates/foo.html.eex Hello &lt; % = @name % &gt; # templates.ex defmodule Templates do use Phoenix.Template , root : &quot;templates&quot; def render ( template , assigns ) do render_template ( template , assigns ) end end Phoenix.Template will define a private function named render_template/2 with one clause per file system template. You are responsible to expose it appropriately, as shown above. In practice, developers rarely use Phoenix.Template directly. Instead they use Phoenix.View which wraps the template functionality and adds some extra conveniences. Options :root - the root template path to find templates :pattern - the wildcard pattern to apply to the root when finding templates. Default &quot;*&quot; :template_engines - a map of template engines extensions to template engine handlers Terminology Here is a quick introduction into Phoenix templates terms: template name - is the name of the template as given by the user, without the template engine extension, for example: &quot;users.html&quot; template path - is the complete path of the template in the filesystem, for example, &quot;path/to/users.html.eex&quot; template root - the directory where templates are defined template engine - a module that receives a template path and transforms its source code into Elixir quoted expressions Custom Template Engines Phoenix supports custom template engines. Engines tell Phoenix how to convert a template path into quoted expressions. See Phoenix.Template.Engine for more information on the API required to be implemented by custom engines. Once a template engine is defined, you can tell Phoenix about it via the template engines option: config :phoenix_view , :template_engines , eex : Phoenix.Template.EExEngine , exs : Phoenix.Template.ExsEngine If you want to support a given engine only on a certain template, you can pass it as an option on use Phoenix.Template : use Phoenix.Template , template_engines : %{ foo : Phoenix.Template.FooEngine } Format encoders Besides template engines, Phoenix has the concept of format encoders. Format encoders work per format and are responsible for encoding a given format to string once the view layer finishes processing. A format encoder must export a function called encode_to_iodata!/1 which receives the rendering artifact and returns iodata. New encoders can be added via the format encoder option: config :phoenix_view , :format_encoders , html : Phoenix.HTML.Engine","ref":"Phoenix.Template.html","title":"Phoenix.Template","type":"module"},{"doc":"Returns a keyword list with all template engines extensions followed by their modules.","ref":"Phoenix.Template.html#engines/0","title":"Phoenix.Template.engines/0","type":"function"},{"doc":"Returns all template paths in a given template root.","ref":"Phoenix.Template.html#find_all/3","title":"Phoenix.Template.find_all/3","type":"function"},{"doc":"Returns the format encoder for the given template name.","ref":"Phoenix.Template.html#format_encoder/1","title":"Phoenix.Template.format_encoder/1","type":"function"},{"doc":"Returns the hash of all template paths in the given root. Used by Phoenix to check if a given root path requires recompilation.","ref":"Phoenix.Template.html#hash/3","title":"Phoenix.Template.hash/3","type":"function"},{"doc":"Converts a module, without the suffix, to a template root. Examples iex&gt; Phoenix.Template . module_to_template_root ( MyApp.UserView , MyApp , &quot;View&quot; ) &quot;user&quot; iex&gt; Phoenix.Template . module_to_template_root ( MyApp.Admin.User , MyApp , &quot;View&quot; ) &quot;admin/user&quot; iex&gt; Phoenix.Template . module_to_template_root ( MyApp.Admin.User , MyApp.Admin , &quot;View&quot; ) &quot;user&quot; iex&gt; Phoenix.Template . module_to_template_root ( MyApp.View , MyApp , &quot;View&quot; ) &quot;&quot; iex&gt; Phoenix.Template . module_to_template_root ( MyApp.View , MyApp.View , &quot;View&quot; ) &quot;&quot;","ref":"Phoenix.Template.html#module_to_template_root/3","title":"Phoenix.Template.module_to_template_root/3","type":"function"},{"doc":"Converts the template path into the template name. Examples iex&gt; Phoenix.Template . template_path_to_name ( ...&gt; &quot;lib/templates/admin/users/show.html.eex&quot; , ...&gt; &quot;lib/templates&quot; ) &quot;admin/users/show.html&quot;","ref":"Phoenix.Template.html#template_path_to_name/2","title":"Phoenix.Template.template_path_to_name/2","type":"function"},{"doc":"","ref":"Phoenix.Template.html#t:name/0","title":"Phoenix.Template.name/0","type":"type"},{"doc":"","ref":"Phoenix.Template.html#t:path/0","title":"Phoenix.Template.path/0","type":"type"},{"doc":"","ref":"Phoenix.Template.html#t:root/0","title":"Phoenix.Template.root/0","type":"type"},{"doc":"The Phoenix engine that handles the .eex extension.","ref":"Phoenix.Template.EExEngine.html","title":"Phoenix.Template.EExEngine","type":"module"},{"doc":"Callback implementation for Phoenix.Template.Engine.compile/2 .","ref":"Phoenix.Template.EExEngine.html#compile/2","title":"Phoenix.Template.EExEngine.compile/2","type":"function"},{"doc":"Specifies the API for adding custom template engines into Phoenix. Engines must implement the compile/2 function, that receives the template file and the template name and outputs the template quoted expression: def compile ( template_path , template_name ) See Phoenix.Template.EExEngine for an example engine implementation.","ref":"Phoenix.Template.Engine.html","title":"Phoenix.Template.Engine","type":"behaviour"},{"doc":"","ref":"Phoenix.Template.Engine.html#c:compile/2","title":"Phoenix.Template.Engine.compile/2","type":"callback"},{"doc":"The Phoenix engine that handles the .exs extension.","ref":"Phoenix.Template.ExsEngine.html","title":"Phoenix.Template.ExsEngine","type":"module"},{"doc":"Callback implementation for Phoenix.Template.Engine.compile/2 .","ref":"Phoenix.Template.ExsEngine.html#compile/2","title":"Phoenix.Template.ExsEngine.compile/2","type":"function"},{"doc":"Defines the view layer of a Phoenix application. The view layer contains conveniences for rendering templates, including support for layouts and encoders per format. Examples Phoenix defines the view template at lib/your_app_web.ex : defmodule YourAppWeb do # ... def view do quote do use Phoenix.View , root : &quot;lib/your_app_web/templates&quot; , namespace : YourAppWeb # Import convenience functions from controllers import Phoenix.Controller , only : [ get_flash : 1 , get_flash : 2 , view_module : 1 , view_template : 1 ] # Use all HTML functionality (forms, tags, etc) use Phoenix.HTML import YourAppWeb.ErrorHelpers import YourAppWeb.Gettext # Alias the Helpers module as Routes alias YourAppWeb.Router.Helpers , as : Routes end end # ... end You can use the definition above to define any view in your application: defmodule YourApp.UserView do use YourAppWeb , :view end Because we have defined the template root to be &quot;lib/your_app_web/templates&quot;, Phoenix.View will automatically load all templates at &quot;your_app_web/templates/user&quot; and include them in the YourApp.UserView . For example, imagine we have the template: # your_app_web/templates/user/index.html.heex Hello &lt; % = @name % &gt; The .heex extension maps to a template engine which tells Phoenix how to compile the code in the file into Elixir source code. After it is compiled, the template can be rendered as: Phoenix.View . render_to_string ( YourApp.UserView , &quot;index.html&quot; , name : &quot;John Doe&quot; ) #=&gt; &quot;Hello John Doe&quot; Differences to Phoenix.LiveView Traditional web applications, that rely on a request/response life cycle, have been typically organized under the Model-View-Controller pattern. In this case, the Controller is responsible for organizing interacting with the model and passing all relevant information to the View for rendering. Phoenix.Controller and Phoenix.View play those roles respectively. Phoenix.LiveView introduces a declarative model where the controller and the view are kept side by side. This empowers Phoenix.LiveView to provide realtime and interactive features under a stateful connection. In other words, you may consider that Phoenix.LiveView abridges both Phoenix.Controller and Phoenix.View responsibilities. Developers do not generally use Phoenix.View from their live views, but LiveView does use Phoenix.View and its features under the scenes. Rendering and formats The main responsibility of a view is to render a template. A template has a name, which also contains a format. For example, in the previous section we have rendered the &quot;index.html&quot; template: Phoenix.View . render_to_string ( YourApp.UserView , &quot;index.html&quot; , name : &quot;John Doe&quot; ) #=&gt; &quot;Hello John Doe&quot; While we got a string at the end, that's not actually what our templates render. Let's take a deeper look: Phoenix.View . render ( YourApp.UserView , &quot;index.html&quot; , name : &quot;John Doe&quot; ) #=&gt; ... This inner representation allows us to separate how templates render and how they are encoded. For example, if you want to render JSON data, we could do so by adding a &quot;show.json&quot; entry to render/2 in our view: defmodule YourApp.UserView do use YourApp.View def render ( &quot;show.json&quot; , %{ user : user } ) do %{ name : user . name , address : user . address } end end Notice that in order to render JSON data, we don't need to explicitly return a JSON string! Instead, we just return data that is encodable to JSON. Now, when we call: Phoenix.View . render_to_string ( YourApp.UserView , &quot;user.json&quot; , user : % User { ... } ) Because the template has the .json extension, Phoenix knows how to encode the map returned for the &quot;user.json&quot; template into an actual JSON payload to be sent over the wire. Phoenix ships with some template engines and format encoders, which can be further configured in the Phoenix application. You can read more about format encoders in Phoenix.Template documentation.","ref":"Phoenix.View.html","title":"Phoenix.View","type":"module"},{"doc":"When used, defines the current module as a main view module. Options :root - the template root to find templates :path - the optional path to search for templates within the :root . Defaults to the underscored view module name. A blank string may be provided to use the :root path directly as the template lookup path :namespace - the namespace to consider when calculating view paths :pattern - the wildcard pattern to apply to the root when finding templates. Default &quot;*&quot; The :root option is required while the :namespace defaults to the first nesting in the module name. For instance, both MyApp.UserView and MyApp.Admin.UserView have namespace MyApp . The :namespace and :path options are used to calculate template lookup paths. For example, if you are in MyApp.UserView and the namespace is MyApp , templates are expected at Path.join(root, &quot;user&quot;) . On the other hand, if the view is MyApp.Admin.UserView , the path will be Path.join(root, &quot;admin/user&quot;) and so on. For explicit root path locations, the :path option can be provided instead. The :root and :path are joined to form the final lookup path. A blank string may be provided to use the :root path directly as the template lookup path. Setting the namespace to MyApp.Admin in the second example will force the template to also be looked up at Path.join(root, &quot;user&quot;) .","ref":"Phoenix.View.html#__using__/1","title":"Phoenix.View.__using__/1","type":"macro"},{"doc":"Renders a template. It expects the view module, the template as a string, and a set of assigns. Notice that this function returns the inner representation of a template. If you want the encoded template as a result, use render_to_iodata/3 instead. Examples Phoenix.View . render ( YourApp.UserView , &quot;index.html&quot; , name : &quot;John Doe&quot; ) #=&gt; {:safe, &quot;Hello John Doe&quot;} Assigns Assigns are meant to be user data that will be available in templates. However, there are keys under assigns that are specially handled by Phoenix, they are: :layout - tells Phoenix to wrap the rendered result in the given layout. See next section Layouts Templates can be rendered within other templates using the :layout option. :layout accepts a tuple of the form {LayoutModule, &quot;template.extension&quot;} . To template that goes inside the layout will be placed in the @inner_content assign: &lt; % = @inner_content % &gt;","ref":"Phoenix.View.html#render/3","title":"Phoenix.View.render/3","type":"function"},{"doc":"Renders a template only if it exists. Note: Using this functionality has been discouraged in recent Phoenix versions, see the &quot;Alternatives&quot; section below. This function works the same as render/3 , but returns nil instead of raising. This is often used with Phoenix.Controller.view_module/1 and Phoenix.Controller.view_template/1 , which must be imported into your views. See the &quot;Examples&quot; section below. Alternatives This function is discouraged. If you need to render something conditionally, the simplest way is to check for an optional function in your views. Consider the case where the application has a sidebar in its layout and it wants certain views to render additional buttons in the sidebar. Inside your sidebar, you could do: &lt; div class = &quot;sidebar&quot; &gt; &lt; % = if function_exported? ( view_module ( @conn ) , :sidebar_additions , 1 ) do &lt; % = view_module ( @conn ) . sidebar_additions ( assigns ) % &gt; &lt; % end % &gt; &lt; / div &gt; If you are using Phoenix.LiveView, you could do similar by accessing the view under @socket : &lt; div class = &quot;sidebar&quot; &gt; &lt; % = if function_exported? ( @socket . view , :sidebar_additions , 1 ) do &lt; % = @socket . view . sidebar_additions ( assigns ) % &gt; &lt; % end % &gt; &lt; / div &gt; Then, in your view or live view, you do: def sidebar_additions ( assigns ) do ~ H \\ &quot;&quot; &quot; ...my additional buttons... \\&quot; &quot; &quot; Using render_existing Consider the case where the application wants to allow entries to be added to a sidebar. This feature could be achieved with: &lt; % = render_existing view_module ( @conn ) , &quot;sidebar_additions.html&quot; , assigns % &gt; Then the module under view_module(@conn) can decide to provide scripts with either a precompiled template, or by implementing the function directly, ie: def render ( &quot;sidebar_additions.html&quot; , _assigns ) do ~H&quot;&quot;&quot; ...my additional buttons... &quot;&quot;&quot; end To use a precompiled template, create a scripts.html.eex file in the templates directory for the corresponding view you want it to render for. For example, for the UserView , create the scripts.html.eex file at your_app_web/templates/user/ .","ref":"Phoenix.View.html#render_existing/3","title":"Phoenix.View.render_existing/3","type":"function"},{"doc":"Renders the given layout passing the given do/end block as @inner_content . This can be useful to implement nested layouts. For example, imagine you have an application layout like this: # layout/app.html.heex &lt; html &gt; &lt; head &gt; &lt; title &gt; Title &lt; / title &gt; &lt; / head &gt; &lt; body &gt; &lt; div class = &quot;menu&quot; &gt; ... &lt; / div &gt; &lt; % = @inner_content % &gt; &lt; / body &gt; This layout is used by many parts of your application. However, there is a subsection of your application that wants to also add a sidebar. Let's call it &quot;blog.html&quot;. You can build on top of the existing layout in two steps. First, define the blog layout: # layout/blog.html.heex &lt; % = render_layout LayoutView , &quot;app.html&quot; , assigns do % &gt; &lt; div class = &quot;sidebar&quot; &gt; ... &lt; / div &gt; &lt; % = @inner_content % &gt; &lt; % end % &gt; And now you can simply use it from your controller: plug :put_layout , &quot;blog.html&quot;","ref":"Phoenix.View.html#render_layout/4","title":"Phoenix.View.render_layout/4","type":"function"},{"doc":"Renders a collection. It receives a collection as an enumerable of structs and returns the rendered collection in a list. This is typically used to render a collection as structured data. For example, to render a list of users to json: render_many ( users , UserView , &quot;show.json&quot; ) which is roughly equivalent to: Enum . map ( users , fn user -&gt; render ( UserView , &quot;show.json&quot; , user : user ) end ) The underlying user is passed to the view and template as :user , which is inferred from the view name. The name of the key in assigns can be customized with the :as option: render_many ( users , UserView , &quot;show.json&quot; , as : :data ) is roughly equivalent to: Enum . map ( users , fn user -&gt; render ( UserView , &quot;show.json&quot; , data : user ) end )","ref":"Phoenix.View.html#render_many/4","title":"Phoenix.View.render_many/4","type":"function"},{"doc":"Renders a single item if not nil. The following: render_one ( user , UserView , &quot;show.json&quot; ) is roughly equivalent to: if user != nil do render ( UserView , &quot;show.json&quot; , user : user ) end The underlying user is passed to the view and template as :user , which is inflected from the view name. The name of the key in assigns can be customized with the :as option: render_one ( user , UserView , &quot;show.json&quot; , as : :data ) is roughly equivalent to: if user != nil do render ( UserView , &quot;show.json&quot; , data : user ) end","ref":"Phoenix.View.html#render_one/4","title":"Phoenix.View.render_one/4","type":"function"},{"doc":"Renders the template and returns iodata.","ref":"Phoenix.View.html#render_to_iodata/3","title":"Phoenix.View.render_to_iodata/3","type":"function"},{"doc":"Renders the template and returns a string.","ref":"Phoenix.View.html#render_to_string/3","title":"Phoenix.View.render_to_string/3","type":"function"},{"doc":"Exception raised when a template cannot be found.","ref":"Phoenix.Template.UndefinedError.html","title":"Phoenix.Template.UndefinedError","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Phoenix.Template.UndefinedError.html#message/1","title":"Phoenix.Template.UndefinedError.message/1","type":"function"},{"doc":"","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Fix dialyzer warnings on template_not_found","ref":"changelog.html#1-1-2-2022-02-02","title":"CHANGELOG - 1.1.2 (2022-02-02)","type":"extras"},{"doc":"Add compile-time dependencies to template engines","ref":"changelog.html#1-1-1-2022-01-31","title":"CHANGELOG - 1.1.1 (2022-01-31)","type":"extras"},{"doc":"Do not add compile time dependencies on arguments given to Phoenix.View and Phoenix.Template Deprecate render_existing/3 in favor of function_exported?/3 checks","ref":"changelog.html#1-1-0-2022-01-06","title":"CHANGELOG - 1.1.0 (2022-01-06)","type":"extras"},{"doc":"Initial release","ref":"changelog.html#1-0-0-2021-07-18","title":"CHANGELOG - 1.0.0 (2021-07-18)","type":"extras"}]